@@ -10,32 +10,43 @@ pub use term::Notation::*;
 /// A type to represent a parsing error.
 #[derive(Debug, PartialEq)]
 pub enum Error {
+    /// an invalid character was encountered
     InvalidCharacter((usize, char)),
+    /// the expression is invalid
     InvalidExpression,
+    /// the expression is empty
     EmptyExpression
 }
 
 #[derive(Debug, PartialEq)]
 enum Token {
+    /// λ or \
     Lambda,
+    /// left parenthesis
     Lparen,
+    /// right parenthesis
     Rparen,
+    /// a number
     Number(usize)
 }
 
 #[derive(Debug, PartialEq)]
 enum CToken {
+    /// an abstraction with a bound variable
     CLambda(String),
+    /// left parenthesis
     CLparen,
+    /// right parenthesis
     CRparen,
+    /// a variable with an identifier
     CName(String)
 }
 
 fn tokenize_dbr(input: &str) -> Result<Vec<Token>, Error> {
-    let mut chars = input.chars().enumerate();
+    let chars = input.chars().enumerate();
     let mut tokens = Vec::new();
 
-    while let Some((i, c)) = chars.next() {
+    for (i, c) in chars {
         match c {
      '\\' | 'λ' => { tokens.push(Lambda) },
             '(' => { tokens.push(Lparen) },
@@ -143,8 +154,11 @@ fn _convert_classic_tokens(tokens: &[CToken], stack: &mut Vec<String>, pos: &mut
 
 #[derive(Debug, PartialEq)]
 enum Expression {
+    /// an abstraction
     Abstraction,
+    /// a sequence of Expressions
     Sequence(Vec<Expression>),
+    /// a variable with a De Bruijn index
     Variable(usize)
 }
 
@@ -167,7 +181,7 @@ fn _get_ast(tokens: &[Token], pos: &mut usize) -> Result<Expression, Error> {
             },
             Lparen => {
                 *pos += 1;
-                let subtree = try!(_get_ast(&tokens, pos));
+                let subtree = try!(_get_ast(tokens, pos));
                 expr.push(subtree);
             },
             Rparen => {
@@ -192,7 +206,7 @@ fn _get_ast(tokens: &[Token], pos: &mut usize) -> Result<Expression, Error> {
 /// # Example
 /// ```
 /// use lambda_calculus::parser::*;
-/// use lambda_calculus::arithmetic::{succ, pred};
+/// use lambda_calculus::church::numerals::{succ, pred};
 ///
 /// assert_eq!(parse(&"λn. λf. λx. n (λg. λh. h (g f)) (λu. x) (λu. u)", Classic), Ok(pred()));
 /// assert_eq!(parse(&"λn.λf.λx.n(λg.λh.h(g f))(λu.x)(λu.u)", Classic), Ok(pred()));
@@ -210,22 +224,20 @@ pub fn parse(input: &str, notation: Notation) -> Result<Term, Error> {
 
     let exprs = try!(if let Sequence(exprs) = ast { Ok(exprs) } else { Err(InvalidExpression) });
 
-    let term = fold_exprs(&exprs, &mut Vec::new(), &mut Vec::new());
-
-    term
+    fold_exprs(&exprs, &mut Vec::new(), &mut Vec::new())
 }
 
 fn fold_exprs(exprs: &[Expression], stack: &mut Vec<Expression>, output: &mut Vec<Term>)
     -> Result<Term, Error>
 {
-    let mut iter = exprs.into_iter();
+    let iter = exprs.into_iter();
 
-    while let Some(ref expr) = iter.next() {
-        match **expr {
+    for expr in iter {
+        match *expr {
             Variable(i) => output.push(Var(i)),
             Abstraction => stack.push(Abstraction),
             Sequence(ref exprs) => {
-                let subexpr = try!(fold_exprs(&exprs, &mut Vec::new(), &mut Vec::new()));
+                let subexpr = try!(fold_exprs(exprs, &mut Vec::new(), &mut Vec::new()));
                 output.push(subexpr)
             }
         }
@@ -243,7 +255,7 @@ fn fold_exprs(exprs: &[Expression], stack: &mut Vec<Expression>, output: &mut Ve
 fn fold_terms(mut terms: Vec<Term>) -> Result<Term, Error> {
     if terms.len() > 1 {
         let fst = terms.remove(0);
-        Ok( terms.into_iter().fold(fst, |acc, t| app(acc, t)) )
+        Ok( terms.into_iter().fold(fst, app) )
     } else if terms.len() == 1 {
         Ok( terms.pop().unwrap() ) // safe; ensured above
     } else {
@@ -316,7 +328,7 @@ mod tests {
     #[test]
     fn parse_y() {
         let y = "λ(λ2(11))(λ2(11))";
-        assert_eq!(parse(&y, DeBruijn).expect("parsing Y failed!"),
+        assert_eq!(parse(&y, DeBruijn).unwrap(),
             abs(
                 app(
                     abs(app(Var(2), app(Var(1), Var(1)))),
@@ -329,7 +341,7 @@ mod tests {
     #[test]
     fn parse_quine() {
         let quine = "λ1((λ11)(λλλλλ14(3(55)2)))1";
-        assert_eq!(parse(&quine, DeBruijn).expect("parsing QUINE failed!"),
+        assert_eq!(parse(&quine, DeBruijn).unwrap(),
             abs(app(app(Var(1), app(abs(app(Var(1), Var(1))), abs(abs(abs(abs(abs(app(app(Var(1),
             Var(4)), app(app(Var(3), app(Var(5), Var(5))), Var(2)))))))))), Var(1)))
         );
@@ -339,7 +351,7 @@ mod tests {
     fn parse_blc() {
         let blc = "(λ11)(λλλ1(λλλλ3(λ5(3(λ2(3(λλ3(λ123)))(4(λ4(λ31(21))))))(1(2(λ12))\
                    (λ4(λ4(λ2(14)))5))))(33)2)(λ1((λ11)(λ11)))";
-        assert_eq!(parse(&blc, DeBruijn).expect("parsing BLC failed!"),
+        assert_eq!(parse(&blc, DeBruijn).unwrap(),
             app(app(abs(app(Var(1), Var(1))), abs(abs(abs(app(app(app(Var(1),
             abs(abs(abs(abs(app(Var(3), abs(app(app(Var(5), app(Var(3), abs(app(app(Var(2),
             app(Var(3), abs(abs(app(Var(3), abs(app(app(Var(1), Var(2)), Var(3)))))))), app(Var(4),
