@@ -35,7 +35,7 @@ pub enum Order {
 }
 
 /// Applies two `Term`s with substitution and variable update, consuming the first one in the
-/// process. It produces an `Error` if the first `Term` is not an `Abs`traction.
+/// process.
 ///
 /// # Example
 /// ```
@@ -43,13 +43,19 @@ pub enum Order {
 /// use lambda_calculus::parser::parse;
 /// use lambda_calculus::term::Notation::DeBruijn;
 ///
+/// // these are valid terms, but be careful with unwraps in your code
 /// let lhs    = parse(&"λλ42(λ13)", DeBruijn).unwrap();
 /// let rhs    = parse(&"λ51", DeBruijn).unwrap();
 /// let result = parse(&"λ3(λ61)(λ1(λ71))", DeBruijn).unwrap();
 ///
 /// assert_eq!(apply(lhs, &rhs), Ok(result));
 /// ```
+/// # Errors
+///
+/// The function will return an error if the `lhs` term is not an `Abs`traction.
 pub fn apply(mut lhs: Term, rhs: &Term) -> Result<Term, Error> {
+    if lhs.unabs_ref().is_err() { return Err(Error::NotAnAbs) }
+
     _apply(&mut lhs, rhs, 0);
 
     lhs.unabs()
@@ -95,7 +101,7 @@ fn update_free_variables(term: &mut Term, added_depth: usize, own_depth: usize)
 /// # Example
 ///
 /// ```
-/// use lambda_calculus::arithmetic::pred;
+/// use lambda_calculus::church::numerals::pred;
 /// use lambda_calculus::reduction::beta;
 /// use lambda_calculus::reduction::Order::NOR;
 ///
@@ -114,7 +120,7 @@ pub fn beta(mut term: Term, order: Order, limit: usize, verbose: bool) -> Term {
 /// # Example
 ///
 /// ```
-/// use lambda_calculus::arithmetic::fac;
+/// use lambda_calculus::church::numerals::fac;
 /// use lambda_calculus::reduction::compare;
 /// use lambda_calculus::reduction::Order::*;
 ///
@@ -122,10 +128,10 @@ pub fn beta(mut term: Term, order: Order, limit: usize, verbose: bool) -> Term {
 ///
 /// // stdout:
 ///
-/// // normal:             46
-/// // applicative:        39
-/// // hybrid normal:      46
-/// // hybrid applicative: 39
+/// // normal:             35
+/// // applicative:        36
+/// // hybrid normal:      35
+/// // hybrid applicative: 30
 /// ```
 pub fn compare(term: &Term, orders: &[Order], verbose: bool) {
     println!("comparing β-reduction strategies for {}:\n", term);
@@ -139,7 +145,6 @@ pub fn compare(term: &Term, orders: &[Order], verbose: bool) {
 
 impl Term {
     /// Applies `self` to another `Term` and performs substitution, consuming `self` in the process.
-    /// It produces an `Error` if `self` is not an `Abs`traction.
     ///
     /// # Example
     /// ```
@@ -152,22 +157,28 @@ impl Term {
     ///
     /// assert_eq!(lhs.apply(&rhs), Ok(result));
     /// ```
+    /// # Errors
+    ///
+    /// The function will return an error if `self` is not an `Abs`traction.
     pub fn apply(self, rhs: &Term) -> Result<Term, Error> {
         apply(self, rhs)
     }
 
-    /// Reduces an `App`lication by substitution and variable update. It produces an `Error` if
-    /// `self` is not an `App`lication.
+    /// Reduces an `App`lication by substitution and variable update.
     ///
     /// # Example
     /// ```
     /// use lambda_calculus::term::{app, abs};
     /// use lambda_calculus::term::Term::Var;
-    /// use lambda_calculus::arithmetic::zero;
+    /// use lambda_calculus::church::numerals::zero;
     /// use lambda_calculus::combinators::i;
     ///
     /// assert_eq!(app(i(), zero()).eval(), Ok(zero()));
     /// ```
+    /// # Errors
+    ///
+    /// The function will return an error if `self` is not an `App`lication or if its left hand
+    /// side term is not an `Abs`traction.
     pub fn eval(self) -> Result<Term, Error> {
         let (lhs, rhs) = try!(self.unapp());
 
@@ -197,7 +208,7 @@ impl Term {
     /// # Example
     ///
     /// ```
-    /// use lambda_calculus::arithmetic::pred;
+    /// use lambda_calculus::church::numerals::pred;
     /// use lambda_calculus::reduction::Order::NOR;
     ///
     /// let mut pred_one = pred().app(1.into());
@@ -239,17 +250,14 @@ impl Term {
     fn beta_cbn(&mut self, depth: u32, limit: usize, count: &mut usize, verbose: bool) {
         if limit != 0 && *count == limit { return }
 
-        match *self {
-            App(_, _) => {
-                self.lhs_ref_mut().unwrap().beta_cbn(depth, limit, count, verbose);
+        if let App(_, _) = *self {
+            self.lhs_mut().unwrap().beta_cbn(depth, limit, count, verbose);
 
-                if self.is_reducible(limit, count) {
-                    self.eval_with_info(depth, count, verbose);
-                    *count += 1;
-                    self.beta_cbn(depth, limit, count, verbose);
-                }
-            },
-            _ => ()
+            if self.is_reducible(limit, count) {
+                self.eval_with_info(depth, count, verbose);
+                *count += 1;
+                self.beta_cbn(depth, limit, count, verbose);
+            }
         }
     }
 
@@ -259,15 +267,15 @@ impl Term {
         match *self {
             Abs(ref mut abstracted) => abstracted.beta_nor(depth + 1, limit, count, verbose),
             App(_, _) => {
-                self.lhs_ref_mut().unwrap().beta_cbn(depth, limit, count, verbose);
+                self.lhs_mut().unwrap().beta_cbn(depth, limit, count, verbose);
 
                 if self.is_reducible(limit, count) {
                     self.eval_with_info(depth, count, verbose);
                     *count += 1;
                     self.beta_nor(depth, limit, count, verbose);
                 } else {
-                    self.lhs_ref_mut().unwrap().beta_nor(depth, limit, count, verbose);
-                    self.rhs_ref_mut().unwrap().beta_nor(depth, limit, count, verbose);
+                    self.lhs_mut().unwrap().beta_nor(depth, limit, count, verbose);
+                    self.rhs_mut().unwrap().beta_nor(depth, limit, count, verbose);
                 }
             },
             _ => ()
@@ -277,18 +285,15 @@ impl Term {
     fn beta_cbv(&mut self, depth: u32, limit: usize, count: &mut usize, verbose: bool) {
         if limit != 0 && *count == limit { return }
 
-        match *self {
-            App(_, _) => {
-                self.lhs_ref_mut().unwrap().beta_cbv(depth, limit, count, verbose);
-                self.rhs_ref_mut().unwrap().beta_cbv(depth, limit, count, verbose);
+        if let App(_, _) = *self {
+            self.lhs_mut().unwrap().beta_cbv(depth, limit, count, verbose);
+            self.rhs_mut().unwrap().beta_cbv(depth, limit, count, verbose);
 
-                if self.is_reducible(limit, count) {
-                    self.eval_with_info(depth, count, verbose);
-                    *count += 1;
-                    self.beta_cbv(depth, limit, count, verbose);
-                }
-            },
-            _ => ()
+            if self.is_reducible(limit, count) {
+                self.eval_with_info(depth, count, verbose);
+                *count += 1;
+                self.beta_cbv(depth, limit, count, verbose);
+            }
         }
     }
 
@@ -298,8 +303,8 @@ impl Term {
         match *self {
             Abs(ref mut abstracted) => abstracted.beta_app(depth + 1, limit, count, verbose),
             App(_, _) => {
-                self.lhs_ref_mut().unwrap().beta_app(depth, limit, count, verbose);
-                self.rhs_ref_mut().unwrap().beta_app(depth, limit, count, verbose);
+                self.lhs_mut().unwrap().beta_app(depth, limit, count, verbose);
+                self.rhs_mut().unwrap().beta_app(depth, limit, count, verbose);
 
                 if self.is_reducible(limit, count) {
                     self.eval_with_info(depth, count, verbose);
@@ -317,15 +322,15 @@ impl Term {
         match *self {
             Abs(ref mut abstracted) => abstracted.beta_hap(depth + 1, limit, count, verbose),
             App(_, _) => {
-                self.lhs_ref_mut().unwrap().beta_cbv(depth, limit, count, verbose);
-                self.rhs_ref_mut().unwrap().beta_hap(depth, limit, count, verbose);
+                self.lhs_mut().unwrap().beta_cbv(depth, limit, count, verbose);
+                self.rhs_mut().unwrap().beta_hap(depth, limit, count, verbose);
 
                 if self.is_reducible(limit, count) {
                     self.eval_with_info(depth, count, verbose);
                     *count += 1;
                     self.beta_hap(depth, limit, count, verbose);
                 } else {
-                    self.lhs_ref_mut().unwrap().beta_hap(depth, limit, count, verbose);
+                    self.lhs_mut().unwrap().beta_hap(depth, limit, count, verbose);
                 }
             },
             _ => ()
@@ -338,7 +343,7 @@ impl Term {
         match *self {
             Abs(ref mut abstracted) => abstracted.beta_hsp(depth + 1, limit, count, verbose),
             App(_, _) => {
-                self.lhs_ref_mut().unwrap().beta_hsp(depth, limit, count, verbose);
+                self.lhs_mut().unwrap().beta_hsp(depth, limit, count, verbose);
 
                 if self.is_reducible(limit, count) {
                     self.eval_with_info(depth, count, verbose);
@@ -356,15 +361,15 @@ impl Term {
         match *self {
             Abs(ref mut abstracted) => abstracted.beta_hno(depth + 1, limit, count, verbose),
             App(_, _) => {
-                self.lhs_ref_mut().unwrap().beta_hsp(depth, limit, count, verbose);
+                self.lhs_mut().unwrap().beta_hsp(depth, limit, count, verbose);
 
                 if self.is_reducible(limit, count) {
                     self.eval_with_info(depth, count, verbose);
                     *count += 1;
                     self.beta_hno(depth, limit, count, verbose)
                 } else {
-                    self.lhs_ref_mut().unwrap().beta_hno(depth, limit, count, verbose);
-                    self.rhs_ref_mut().unwrap().beta_hno(depth, limit, count, verbose);
+                    self.lhs_mut().unwrap().beta_hno(depth, limit, count, verbose);
+                    self.rhs_mut().unwrap().beta_hno(depth, limit, count, verbose);
                 }
             },
             _ => ()
@@ -391,10 +396,9 @@ mod tests {
     use super::*;
     use parser::parse;
     use combinators::{i, omm};
-    use arithmetic::fac;
+    use church::numerals::fac;
     use std::thread;
 
-
     #[test]
     fn normal_order() {
         let reduces_instantly = parse(&"(λλ1)((λλλ((32)1))(λλ2))", DeBruijn).unwrap();
