@@ -7,20 +7,25 @@ use std::fmt;
 use std::borrow::Cow;
 use std::char::from_u32;
 
-/// Set to `true` for λ or `false` for \ when displaying lambda terms. The default is `true`.
-pub const PRETTY_LAMBDA: bool = true;
+/// The character used to display lambda abstractions (a backslash).
+#[cfg(feature = "backslash_lambda")]
+pub const LAMBDA: char = '\\';
+
+/// The character used to display lambda abstractions. The default is the Greek letter 'λ', but it
+/// can also be set to a '\' (backslash) using `features = ["backslash_lambda"]`.
+#[cfg(not(feature = "backslash_lambda"))]
+pub const LAMBDA: char = 'λ';
 
 /// The notation used for parsing and displaying purposes.
 ///
 /// # Example
 /// ```
-/// use lambda_calculus::term::Notation::*;
-/// use lambda_calculus::arithmetic::succ;
+/// use lambda_calculus::church::numerals::succ;
 ///
-/// assert_eq!(&format!(  "{}", succ()), "λa.λb.λc.b (a b c)");
-/// assert_eq!(&format!("{:?}", succ()), "λλλ2(321)");
+/// assert_eq!(&format!(  "{}", succ()), "λa.λb.λc.b (a b c)"); // Classic notation
+/// assert_eq!(&format!("{:?}", succ()), "λλλ2(321)");          // DeBruijn index notation
 /// ```
-#[derive(Debug, PartialEq)]
+#[derive(Debug, PartialEq, Clone, Copy)]
 pub enum Notation {
     /// classic lambda calculus notation; the default `fmt::Display` mode
     Classic,
@@ -30,7 +35,7 @@ pub enum Notation {
 
 /// A lambda term that is either a variable with a De Bruijn index, an abstraction over a term or
 /// an applicaction of one term to another.
-#[derive(PartialEq, Clone)]
+#[derive(PartialEq, Clone, Hash, Eq)]
 pub enum Term {
     /// a variable
     Var(usize),
@@ -43,11 +48,17 @@ pub enum Term {
 /// An error that can be returned when an inapplicable function is applied to a term.
 #[derive(Debug, PartialEq)]
 pub enum Error {
+    /// the term is not a variable
     NotAVar,
+    /// the term is not an abstraction
     NotAnAbs,
+    /// the term is not an application
     NotAnApp,
+    /// the term is not a Church number
     NotANum,
+    /// the term is not a Church pair
     NotAPair,
+    /// the term is not a Church list
     NotAList
 }
 
@@ -56,10 +67,9 @@ impl Term {
     ///
     /// # Example
     /// ```
-    /// use lambda_calculus::term::Term::*;
-    /// use lambda_calculus::arithmetic::{zero, succ};
+    /// use lambda_calculus::term::*;
     ///
-    /// assert_eq!(succ().app(zero()), App(Box::new(succ()), Box::new(zero())));
+    /// assert_eq!(Var(1).app(Var(2)), App(Box::new(Var(1)), Box::new(Var(2))));
     /// ```
     pub fn app(self, argument: Term) -> Term { App(Box::new(self), Box::new(argument)) }
 
@@ -67,148 +77,148 @@ impl Term {
     ///
     /// # Example
     /// ```
-    /// use lambda_calculus::term::Term::*;
+    /// use lambda_calculus::term::*;
     ///
     /// assert_eq!(Var(1).unvar(), Ok(1));
     /// ```
+    /// # Errors
+    ///
+    /// The function will return an error if `self` is not a `Var`iable.
     pub fn unvar(self) -> Result<usize, Error> {
-        match self {
-            Var(n) => Ok(n),
-            _ => Err(NotAVar)
-        }
+        if let Var(n) = self { Ok(n) } else { Err(NotAVar) }
     }
 
     /// Returns a reference to a variable's index.
     ///
     /// # Example
     /// ```
-    /// use lambda_calculus::term::Term::*;
+    /// use lambda_calculus::term::*;
     ///
     /// assert_eq!(Var(1).unvar_ref(), Ok(&1));
     /// ```
+    /// # Errors
+    ///
+    /// The function will return an error if `self` is not a `Var`iable.
     pub fn unvar_ref(&self) -> Result<&usize, Error> {
-        match *self {
-            Var(ref n) => Ok(n),
-            _ => Err(NotAVar)
-        }
+        if let Var(ref n) = *self { Ok(n) } else { Err(NotAVar) }
     }
 
     /// Returns a mutable reference to a variable's index.
     ///
     /// # Example
     /// ```
-    /// use lambda_calculus::term::Term::*;
+    /// use lambda_calculus::term::*;
     ///
-    /// assert_eq!(Var(1).unvar_ref_mut(), Ok(&mut 1));
+    /// assert_eq!(Var(1).unvar_mut(), Ok(&mut 1));
     /// ```
-    pub fn unvar_ref_mut(&mut self) -> Result<&mut usize, Error> {
-        match *self {
-            Var(ref mut n) => Ok(n),
-            _ => Err(NotAVar)
-        }
+    /// # Errors
+    ///
+    /// The function will return an error if `self` is not a `Var`iable.
+    pub fn unvar_mut(&mut self) -> Result<&mut usize, Error> {
+        if let Var(ref mut n) = *self { Ok(n) } else { Err(NotAVar) }
     }
 
     /// Consumes an abstraction and returns its underlying term.
     ///
     /// # Example
     /// ```
-    /// use lambda_calculus::term::Term::*;
-    /// use lambda_calculus::term::abs;
+    /// use lambda_calculus::term::*;
     ///
     /// assert_eq!(abs(Var(1)).unabs(), Ok(Var(1)));
     /// ```
+    /// # Errors
+    ///
+    /// The function will return an error if `self` is not an `Abs`traction.
     pub fn unabs(self) -> Result<Term, Error> {
-        match self {
-            Abs(x) => Ok(*x),
-            _ => Err(NotAnAbs)
-        }
+        if let Abs(x) = self { Ok(*x) } else { Err(NotAnAbs) }
     }
 
     /// Returns a reference to an abstraction's underlying term.
     ///
     /// # Example
     /// ```
-    /// use lambda_calculus::term::Term::*;
-    /// use lambda_calculus::term::abs;
+    /// use lambda_calculus::term::*;
     ///
     /// assert_eq!(abs(Var(1)).unabs_ref(), Ok(&Var(1)));
     /// ```
+    /// # Errors
+    ///
+    /// The function will return an error if `self` is not an `Abs`traction.
     pub fn unabs_ref(&self) -> Result<&Term, Error> {
-        match *self {
-            Abs(ref x) => Ok(x),
-            _ => Err(NotAnAbs)
-        }
+        if let Abs(ref x) = *self { Ok(x) } else { Err(NotAnAbs) }
     }
 
     /// Returns a mutable reference to an abstraction's underlying term.
     ///
     /// # Example
     /// ```
-    /// use lambda_calculus::term::Term::*;
-    /// use lambda_calculus::term::abs;
+    /// use lambda_calculus::term::*;
     ///
-    /// assert_eq!(abs(Var(1)).unabs_ref_mut(), Ok(&mut Var(1)));
+    /// assert_eq!(abs(Var(1)).unabs_mut(), Ok(&mut Var(1)));
     /// ```
-    pub fn unabs_ref_mut(&mut self) -> Result<&mut Term, Error> {
-        match *self {
-            Abs(ref mut x) => Ok(x),
-            _ => Err(NotAnAbs)
-        }
+    /// # Errors
+    ///
+    /// The function will return an error if `self` is not an `Abs`traction.
+    pub fn unabs_mut(&mut self) -> Result<&mut Term, Error> {
+        if let Abs(ref mut x) = *self { Ok(x) } else { Err(NotAnAbs) }
     }
 
     /// Consumes an application and returns a pair containing its underlying terms.
     ///
     /// # Example
     /// ```
-    /// use lambda_calculus::term::Term::*;
+    /// use lambda_calculus::term::*;
     ///
     /// assert_eq!(Var(1).app(Var(2)).unapp(), Ok((Var(1), Var(2))));
     /// ```
+    /// # Errors
+    ///
+    /// The function will return an error if `self` is not an `App`lication.
     pub fn unapp(self) -> Result<(Term, Term), Error> {
-        match self {
-            App(lhs, rhs) => Ok((*lhs, *rhs)),
-            _ => Err(NotAnApp)
-        }
+        if let App(lhs, rhs) = self { Ok((*lhs, *rhs)) } else { Err(NotAnApp) }
     }
 
     /// Returns a pair containing references to an application's underlying terms.
     ///
     /// # Example
     /// ```
-    /// use lambda_calculus::term::Term::*;
+    /// use lambda_calculus::term::*;
     ///
     /// assert_eq!(Var(1).app(Var(2)).unapp_ref(), Ok((&Var(1), &Var(2))));
     /// ```
+    /// # Errors
+    ///
+    /// The function will return an error if `self` is not an `App`lication.
     pub fn unapp_ref(&self) -> Result<(&Term, &Term), Error> {
-        match *self {
-            App(ref lhs, ref rhs) => Ok((lhs, rhs)),
-            _ => Err(NotAnApp)
-        }
+        if let App(ref lhs, ref rhs) = *self { Ok((lhs, rhs)) } else { Err(NotAnApp) }
     }
 
     /// Returns a pair containing mutable references to an application's underlying terms.
     ///
     /// # Example
     /// ```
-    /// use lambda_calculus::term::Term::*;
+    /// use lambda_calculus::term::*;
     ///
-    /// assert_eq!(Var(1).app(Var(2)).unapp_ref_mut(), Ok((&mut Var(1), &mut Var(2))));
+    /// assert_eq!(Var(1).app(Var(2)).unapp_mut(), Ok((&mut Var(1), &mut Var(2))));
     /// ```
-    pub fn unapp_ref_mut(&mut self) -> Result<(&mut Term, &mut Term), Error> {
-        match *self {
-            App(ref mut lhs, ref mut rhs) => Ok((lhs, rhs)),
-            _ => Err(NotAnApp)
-        }
+    /// # Errors
+    ///
+    /// The function will return an error if `self` is not an `App`lication.
+    pub fn unapp_mut(&mut self) -> Result<(&mut Term, &mut Term), Error> {
+        if let App(ref mut lhs, ref mut rhs) = *self { Ok((lhs, rhs)) } else { Err(NotAnApp) }
     }
 
     /// Returns the left-hand side term of an application. Consumes `self`.
     ///
     /// # Example
     /// ```
-    /// use lambda_calculus::term::Term::*;
+    /// use lambda_calculus::term::*;
     ///
     /// assert_eq!(Var(1).app(Var(2)).lhs(), Ok(Var(1)));
     /// ```
+    /// # Errors
+    ///
+    /// The function will return an error if `self` is not an `App`lication.
     pub fn lhs(self) -> Result<Term, Error> {
         if let Ok((lhs, _)) = self.unapp() { Ok(lhs) } else { Err(NotAnApp) }
     }
@@ -217,10 +227,13 @@ impl Term {
     ///
     /// # Example
     /// ```
-    /// use lambda_calculus::term::Term::*;
+    /// use lambda_calculus::term::*;
     ///
     /// assert_eq!(Var(1).app(Var(2)).lhs_ref(), Ok(&Var(1)));
     /// ```
+    /// # Errors
+    ///
+    /// The function will return an error if `self` is not an `App`lication.
     pub fn lhs_ref(&self) -> Result<&Term, Error> {
         if let Ok((lhs, _)) = self.unapp_ref() { Ok(lhs) } else { Err(NotAnApp) }
     }
@@ -229,22 +242,25 @@ impl Term {
     ///
     /// # Example
     /// ```
-    /// use lambda_calculus::term::Term::*;
+    /// use lambda_calculus::term::*;
     ///
-    /// assert_eq!(Var(1).app(Var(2)).lhs_ref_mut(), Ok(&mut Var(1)));
+    /// assert_eq!(Var(1).app(Var(2)).lhs_mut(), Ok(&mut Var(1)));
     /// ```
-    pub fn lhs_ref_mut(&mut self) -> Result<&mut Term, Error> {
-        if let Ok((lhs, _)) = self.unapp_ref_mut() { Ok(lhs) } else { Err(NotAnApp) }
+    pub fn lhs_mut(&mut self) -> Result<&mut Term, Error> {
+        if let Ok((lhs, _)) = self.unapp_mut() { Ok(lhs) } else { Err(NotAnApp) }
     }
 
     /// Returns the right-hand side term of an application. Consumes `self`.
     ///
     /// # Example
     /// ```
-    /// use lambda_calculus::term::Term::*;
+    /// use lambda_calculus::term::*;
     ///
     /// assert_eq!(Var(1).app(Var(2)).rhs(), Ok(Var(2)));
     /// ```
+    /// # Errors
+    ///
+    /// The function will return an error if `self` is not an `App`lication.
     pub fn rhs(self) -> Result<Term, Error> {
         if let Ok((_, rhs)) = self.unapp() { Ok(rhs) } else { Err(NotAnApp) }
     }
@@ -253,10 +269,13 @@ impl Term {
     ///
     /// # Example
     /// ```
-    /// use lambda_calculus::term::Term::*;
+    /// use lambda_calculus::term::*;
     ///
     /// assert_eq!(Var(1).app(Var(2)).rhs_ref(), Ok(&Var(2)));
     /// ```
+    /// # Errors
+    ///
+    /// The function will return an error if `self` is not an `App`lication.
     pub fn rhs_ref(&self) -> Result<&Term, Error> {
         if let Ok((_, rhs)) = self.unapp_ref() { Ok(rhs) } else { Err(NotAnApp) }
     }
@@ -265,12 +284,15 @@ impl Term {
     ///
     /// # Example
     /// ```
-    /// use lambda_calculus::term::Term::*;
+    /// use lambda_calculus::term::*;
     ///
-    /// assert_eq!(Var(1).app(Var(2)).rhs_ref_mut(), Ok(&mut Var(2)));
+    /// assert_eq!(Var(1).app(Var(2)).rhs_mut(), Ok(&mut Var(2)));
     /// ```
-    pub fn rhs_ref_mut(&mut self) -> Result<&mut Term, Error> {
-        if let Ok((_, rhs)) = self.unapp_ref_mut() { Ok(rhs) } else { Err(NotAnApp) }
+    /// # Errors
+    ///
+    /// The function will return an error if `self` is not an `App`lication.
+    pub fn rhs_mut(&mut self) -> Result<&mut Term, Error> {
+        if let Ok((_, rhs)) = self.unapp_mut() { Ok(rhs) } else { Err(NotAnApp) }
     }
 }
 
@@ -278,20 +300,18 @@ impl Term {
 ///
 /// # Example
 /// ```
-/// use lambda_calculus::term::Term::*;
-/// use lambda_calculus::term::abs;
+/// use lambda_calculus::term::*;
 ///
 /// assert_eq!(abs(Var(1)), Abs(Box::new(Var(1))));
 /// ```
 pub fn abs(term: Term) -> Term { Abs(Box::new(term)) }
 
-/// Produces an `App`lication of 2 given `Term`s without any reduction, consuming them in the
+/// Produces an `App`lication of two given `Term`s without any reduction, consuming them in the
 /// process.
 ///
 /// # Example
 /// ```
-/// use lambda_calculus::term::Term::*;
-/// use lambda_calculus::term::app;
+/// use lambda_calculus::term::*;
 ///
 /// assert_eq!(app(Var(1), Var(2)), App(Box::new(Var(1)), Box::new(Var(2))));
 /// ```
@@ -315,7 +335,7 @@ pub fn show_precedence_cla(term: &Term, context_precedence: usize, depth: u32) -
         },
         Abs(ref t) => {
             let ret = {
-                format!("{}{}.{}", if PRETTY_LAMBDA { 'λ' } else { '\\' },
+                format!("{}{}.{}", LAMBDA,
                     from_u32(depth + 97).expect("error while printing term"),
                     show_precedence_cla(t, 0, depth + 1)
                 )
@@ -343,7 +363,7 @@ pub fn show_precedence_dbr(term: &Term, context_precedence: usize, depth: u32) -
             format!("{:X}", i)
         },
         Abs(ref t) => {
-            let ret = format!("{}{:?}", if PRETTY_LAMBDA { 'λ' } else { '\\' }, t);
+            let ret = format!("{}{:?}", LAMBDA, t);
             parenthesize_if(&ret, context_precedence > 1).into()
         },
         App(ref t1, ref t2) => {
@@ -368,8 +388,7 @@ fn parenthesize_if(input: &str, condition: bool) -> Cow<str> {
 /// ```
 /// # #[macro_use] extern crate lambda_calculus;
 /// # fn main() {
-/// use lambda_calculus::term::Term::*;
-/// use lambda_calculus::term::app;
+/// use lambda_calculus::term::*;
 ///
 /// assert_eq!(app!(Var(1), Var(2), Var(3)), Var(1).app(Var(2)).app(Var(3)));
 /// # }
@@ -387,44 +406,40 @@ macro_rules! app {
 
 #[cfg(test)]
 mod tests {
-    use super::{Var, PRETTY_LAMBDA};
-    use super::Notation::DeBruijn;
-    use parser::parse;
-    use arithmetic::{zero, succ, pred};
+    use super::*;
 
     #[test]
     fn app_macro() {
-        assert_eq!(app!(succ(), app!(Var(1), Var(2), Var(3))),
-                   succ().app(Var(1).app(Var(2)).app(Var(3)))
+        assert_eq!(app!(Var(4), app!(Var(1), Var(2), Var(3))),
+                   Var(4).app(Var(1).app(Var(2)).app(Var(3)))
         );
     }
 
     #[test]
     fn open_term_display() {
-        assert_eq!(&format!("{}", parse("λ2",  DeBruijn).unwrap()), "λa.b");
-        assert_eq!(&format!("{}", parse("λ3",  DeBruijn).unwrap()), "λa.c");
-        assert_eq!(&format!("{}", parse("λλ3", DeBruijn).unwrap()), "λa.λb.c");
-        assert_eq!(&format!("{}", parse("λλ4", DeBruijn).unwrap()), "λa.λb.d");
+        assert_eq!(&format!("{}",     abs(Var(2))) ,  "λa.b");
+        assert_eq!(&format!("{}",     abs(Var(3))) ,  "λa.c");
+        assert_eq!(&format!("{}", abs(abs(Var(3)))), "λa.λb.c");
+        assert_eq!(&format!("{}", abs(abs(Var(4)))), "λa.λb.d");
     }
 
     #[test]
     fn display_modes() {
-        if PRETTY_LAMBDA {
-            assert_eq!(&format!("{}", zero()), "λa.λb.b");
-            assert_eq!(&format!("{}", succ()), "λa.λb.λc.b (a b c)");
-            assert_eq!(&format!("{}", pred()), "λa.λb.λc.a (λd.λe.e (d b)) (λd.c) (λd.d)");
+        let zero = abs(abs(Var(1)));
+        let succ = abs(abs(abs(app(Var(2), app!(Var(3), Var(2), Var(1))))));
+        let pred = abs(abs(abs(app!(
+            Var(3),
+            abs(abs(app(Var(1), app(Var(2), Var(4))))),
+            abs(Var(2)),
+            abs(Var(1))
+        ))));
 
-            assert_eq!(&format!("{:?}", zero()), "λλ1");
-            assert_eq!(&format!("{:?}", succ()), "λλλ2(321)");
-            assert_eq!(&format!("{:?}", pred()), "λλλ3(λλ1(24))(λ2)(λ1)");
-        } else {
-            assert_eq!(&format!("{}", zero()), r#"\a.\b.b"#);
-            assert_eq!(&format!("{}", succ()), r#"\a.\b.\c.b (a b c)"#);
-            assert_eq!(&format!("{}", pred()), r#"\a.\b.\c.a (\d.\e.e (d b)) (\d.c) (\d.d)"#);
+        assert_eq!(&format!("{}", zero), "λa.λb.b");
+        assert_eq!(&format!("{}", succ), "λa.λb.λc.b (a b c)");
+        assert_eq!(&format!("{}", pred), "λa.λb.λc.a (λd.λe.e (d b)) (λd.c) (λd.d)");
 
-            assert_eq!(&format!("{:?}", zero()), r#"\\1"#);
-            assert_eq!(&format!("{:?}", succ()), r#"\\\2(321)"#);
-            assert_eq!(&format!("{:?}", pred()), r#"\\\3(\\1(24))(\2)(\1)"#);
-        }
+        assert_eq!(&format!("{:?}", zero), "λλ1");
+        assert_eq!(&format!("{:?}", succ), "λλλ2(321)");
+        assert_eq!(&format!("{:?}", pred), "λλλ3(λλ1(24))(λ2)(λ1)");
     }
 }
